

import matplotlib.pyplot as plt
import numpy as np

from amrs.misc.chem import ppm_to_mass, kgs_to_Mtyr


COLORS = ['#1f77b4', '#ff7f0e', '#2ca02c', '#d62728', '#9467bd',
          '#8c564b', '#e377c2', '#7f7f7f', '#bcbd22', '#17becf']



def init_mass_balance_plot(tracer, distances, delta, emissions, q_true):
    xvalues = (distances[1:] - delta / 2) / 1e3 # m -> km

    fig = plt.figure()

    ax = plt.gca()
    ax2 = plt.twinx()

    ax.set_position([0.10, 0.10, 0.81, 0.87])
    ax2.set_position([0.10, 0.10, 0.81, 0.87])

    lines = []

    lines.append(
        ax2.fill_betweenx([0,80], [-20,-20], [+20,+20], color='#dddddd',
                          label='city area')
    )

    lines += ax.plot([xvalues[0]-delta/2e3, xvalues[-1]+delta/2e3],
                     [q_true, q_true], 'k-', label='true emissions (10:30 UTC)')

    label = 'range of true emissions (6-12 UTC)'
    lines.append(
        ax.fill_between([xvalues[0]-delta/2e3, xvalues[-1]+delta/2e3],
                        [min(emissions), min(emissions)],
                        [max(emissions), max(emissions)],
                        label=label, color='gray')
    )

    ax.set_xlabel('along plume distance from source (km)')

    if tracer == 'CO2':
        ax.set_ylabel('CO$_2$ flux (Mt CO$_2$ yr$^{-1}$)')
    else:
        ax.set_ylabel('NO$_x$ flux (kt NO$_2$ yr$^{-1}$)')

    return ax, ax2, lines



def add_mass_balance_plot(i, ax, ax2, tracer, distances, delta, flux, flux_std,
                          ueff, lines=None, add_wind=False,
                          dq=np.nan, fit=None):

    if lines is None:
        lines = []

    xvalues = (distances[1:] - delta / 2) / 1e3 # m -> km

    if add_wind:
        lines.append(
            ax2.bar(xvalues, ueff, width=0.9*delta/1e3,
                    label='wind speed in plume (m s$^{-1}$)',
                    color='#aa8888')
        )

    factor = 1.0 if  tracer == 'CO2' else 1e3

    labels = [
        'method errors (%s)',
        'wind error (%s)',
        'instrument noise (%s)',
        'background errors (%s)',
        'combined errors (%s)',
    ]
    marker = ['o', 's', 'd', 'v', '^'][i]

    extra = '$\Delta$Q = %+.1f %st yr$^{-1}$' % (dq, 'k' if tracer ==
                                         'NO2' else 'M')
    label = labels[i] % extra

    lines.append( ax.errorbar(xvalues, factor * kgs_to_Mtyr(flux),
                         factor * kgs_to_Mtyr(flux_std),
                         ls='', marker=marker,
                         label=label, color=COLORS[i])
                )
    ax.plot(xvalues, fit, label='__none__',
            color=COLORS[i])

    ax.set_xlim(min(distances)/1e3, max(distances)/1e3)

    return lines



def finalize_mass_balance_plot(ax, ax2, tracer, lines):

    ax.set_ylim(0, 80 if tracer == 'NO2' else 80)

    ax2.set_ylim(0,50)
    ax2.set_ylabel('wind speed (m s$^{-1}$)')

    ax.set_zorder(ax2.get_zorder()+1)
    ax.patch.set_visible(False)

    ax.legend(lines, [l.get_label() for l in lines],
              loc='upper left', fontsize='small', ncol=1)



