

"""
Extracting wind information from 4D wind fields.
"""

from datetime import datetime
import os

import numpy as np
import xarray

import smartcarb.config as cfg
import smartcarb.io as io
import smartcarb.misc as misc

import level2


def load_wind_field(filename):

    lon, lat, u, v = io.read_fields(filename, ['lon', 'lat', 'U_10M',
                                                 'V_10M'])

    # TODO: read different height levels and interpolate

    return lon, lat, u, v



def extract_wind(data, u, v):
    """
    Extract wind(lon,lat) at locations x,y.
    """
    u_sat, v_sat = level2.sum_satellite_pixels([u, v], data.lonc.values,
                                               data.latc.values)

    data['U_10M'] = xarray.DataArray(u_sat)
    data['V_10M'] = xarray.DataArray(v_sat)

    return data



def add_wind(data, tracer='CO2'):

    time = data.attrs['time']

    if time < datetime(2015,7,1):
        starttime = datetime(2015,1,1)
    else:
        starttime = datetime(2015,7,1)

    # time-varying fields
    path = os.path.join(cfg.DATA_PATH, starttime.strftime('cosmo_%Y-%m'))
    filename = os.path.join(path, time.strftime('lffd%Y%m%d%H.nc'))

    tracer_name = {
        'CO2': 'CO2_BV',
        'NO2': 'NOX_BV',
    }[tracer]
    fields = ['lon', 'lat', 'U', 'V', tracer_name, 'U_10M', 'V_10M']
    lon, lat, u, v, c, u10, v10 = io.read_fields(filename, fields, time=time)

    # constant fields
    filename = os.path.join(path, starttime.strftime('lffd%Y%m%d%Hc.nc'))
    hhl = io.read_fields(filename, ['HHL'])[0]

    # effective wind vectors
    u_eff = np.sum(u * c, axis=0) / np.sum(c, axis=0)
    v_eff = np.sum(v * c, axis=0) / np.sum(c, axis=0)

    u_eff = np.ma.array(u_eff, mask=np.isnan(u_eff))
    v_eff = np.ma.array(v_eff, mask=np.isnan(v_eff))


    fields = level2.sum_satellite_pixels([u_eff, v_eff, u10, v10], data.lonc.values,
                                         data.latc.values)

    dims = ('nobs', 'nrows')
    data['u_eff'] = xarray.DataArray(fields[0], dims=dims, attrs={'units': 'm s-1'})
    data['v_eff'] = xarray.DataArray(fields[1], dims=dims, attrs={'units': 'm s-1'})
    data['u_10m'] = xarray.DataArray(fields[2], dims=dims, attrs={'units': 'm s-1'})
    data['v_10m'] = xarray.DataArray(fields[3], dims=dims, attrs={'units': 'm s-1'})


    return data


    fields = np.concatenate([hhl[50:], u[50:], v[50:], c[50:]])

    fields = level2.sum_satellite_pixels(fields, data.lonc.values,
                                         data.latc.values)

    hhl = fields[0:10]
    u = fields[10:20]
    v = fields[20:30]
    c = fields[30:]

    dims = ('nlevels', 'nobs', 'nrows')
    data['HHL'] = xarray.DataArray(u, dims=dims, attrs={'units': 'm'})

    dims = ('nlayers', 'nobs', 'nrows')
    data['u'] = xarray.dataarray(u, dims=dims, attrs={'units': 'm s-1'})
    data['v'] = xarray.dataarray(v, dims=dims, attrs={'units': 'm s-1'})
    data['CO2_profile'] = xarray.DataArray(c, dims=dims,
                                           attrs={'long_name': 'CO2 Berlin tracer',
                                                  'units': 'kg kg-1'})


    return data



